package com.aurellem.capture.hello;

import java.io.File;

import com.aurellem.capture.IsoTimer;
import com.aurellem.capture.MultiListener;
import com.aurellem.capture.WaveFileWriter;
import com.jme3.app.SimpleApplication;
import com.jme3.audio.AudioNode;
import com.jme3.audio.Listener;
import com.jme3.input.controls.ActionListener;
import com.jme3.input.controls.MouseButtonTrigger;
import com.jme3.material.Material;
import com.jme3.math.ColorRGBA;
import com.jme3.math.Quaternion;
import com.jme3.math.Vector3f;
import com.jme3.scene.Geometry;
import com.jme3.scene.shape.Box;
import com.jme3.system.AppSettings;

/** Sample 11 - playing 3D audio. */
public class HelloAudio extends SimpleApplication {
	  
  private AudioNode audio_gun;
  private AudioNode audio_nature;
  private Geometry player;
  private Listener auxListener = new Listener(); 
  public File data1 = new File("/home/r/tmp/data1.wav");
  public File data2 = new File("/home/r/tmp/data2.wav");
  public File data3 = new File("/home/r/tmp/data3.wav");
   
  private File makeTarget(int n){
	  	return new File("/home/r/tmp/assload-" + n + ".wav");
  }
  
  
  public static void main(String[] args) {

	 // Logger.getLogger("com.jme3").setLevel(Level.OFF);
	  
	HelloAudio app = new HelloAudio();
	AppSettings settings = new AppSettings(true);
	
	settings.setAudioRenderer("Send");
	app.setSettings(settings);
	app.setShowSettings(false);
	app.start();
	app.setPauseOnLostFocus(false);
  }

  @Override
  public void simpleInitApp() {
	this.setTimer(new IsoTimer(60));
	
	
	if (this.audioRenderer instanceof MultiListener){
		MultiListener rf = (MultiListener)this.audioRenderer;
		
		for (int n = 0; n < 0; n++){
			Listener zzz = new Listener();
			rf.addListener(zzz);
			rf.registerSoundProcessor(
						zzz, new WaveFileWriter(makeTarget(n)));
		}
		Listener listener3 = new Listener();
		rf.addListener(auxListener);
		rf.addListener(listener3);
		rf.registerSoundProcessor(new WaveFileWriter(data1));
		rf.registerSoundProcessor(auxListener, new WaveFileWriter(data2));
		rf.registerSoundProcessor(listener3, new WaveFileWriter(data3));
	}
    flyCam.setMoveSpeed(40);
    
    /** just a blue box floating in space */
    Box box1 = new Box(Vector3f.ZERO, 1, 1, 1);
    player = new Geometry("Player", box1);
    Material mat1 = new Material(assetManager, "Common/MatDefs/Misc/Unshaded.j3md");
    mat1.setColor("Color", ColorRGBA.Blue);
    player.setMaterial(mat1);
    rootNode.attachChild(player);
    
    /** custom init methods, see below */
    initKeys();
    initAudio();
	
	this.audioRenderer.playSource(audio_gun);

	

  }

  /** We create two audio nodes. */ 
  private void initAudio() {
	//audioRenderer.setEnvironment(Environment.Cavern);
    /* gun shot sound is to be triggered by a mouse click. */
	audio_gun = new AudioNode(audioRenderer, assetManager, "Sound/Effects/Gun.wav", false);
	//audio_gun = new AudioNode(assetManager, "Sound/Effects/dream.wav", false, false);
    audio_gun.setLooping(false);
    audio_gun.setVolume(2);
    audio_gun.setPositional(true);


    /* nature sound - keeps playing in a loop. */
    audio_nature = new AudioNode(assetManager, "Sound/Environment/Nature.ogg", false, false);
    audio_nature.setLooping(true);
    audio_nature.setPositional(true);
    audio_nature.setLocalTranslation(Vector3f.ZERO.clone());
    audio_nature.setVolume(3);
    audio_nature.updateGeometricState();
  }

  /** Declaring the "Shoot" action, and
   *  mapping it to a trigger (mouse click). */
  private void initKeys() {
    inputManager.addMapping("Shoot", new MouseButtonTrigger(0));
    inputManager.addListener(actionListener, "Shoot");
  }

  /** Defining the "Shoot" action: Play a gun sound. */
  private ActionListener actionListener = new ActionListener() {
    @Override
    public void onAction(String name, boolean keyPressed, float tpf) {
      if (name.equals("Shoot") && !keyPressed) {
        audioRenderer.playSource(audio_gun); // play once!
      }
    }
  };

  /** Move the listener with the camera - for 3D audio. */
  @Override
  public void simpleUpdate(float tpf) {
	Vector3f loc = cam.getLocation();
	Quaternion rot = cam.getRotation();
    listener.setLocation(loc);
    listener.setRotation(rot);
    auxListener.setLocation(loc);
    auxListener.setRotation(rot);
    if (audio_gun.getStatus() == AudioNode.Status.Stopped){
    	System.out.println("I'm Stopped!");
    	this.requestClose(false);
    }
    	
    	
  }

}
